#
# Copyright (c) nexB Inc. and others.
# Visit https://github.com/nexB/typecode/
# Visit https://nexb.com https://aboutcode.org and
#  https://github.com/nexB/scancode-toolkit/ for support
# ScanCode is a trademark of nexB Inc.
#
# SPDX-License_identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from functools import partial
import bz2
import gzip
import os
import tarfile
import zipfile


"""
Utilities to detect is a file is extractible. We prefer using extractcode
support if available, otherwise we use the standard library archive
and compressed files support as available.
"""


def _is_compressed(location, opener):
    if os.path.isfile(location):
        try:
            with opener(location) as comp:
                comp.read()
            return True
        except Exception:
            return False


is_gzipfile = partial(_is_compressed, opener=gzip.open)

is_bz2file = partial(_is_compressed, opener=bz2.BZ2File)

try:
    import lzma
    is_lzmafile = partial(_is_compressed, opener=lzma.open)
except ImportError:
    is_lzmafile = lambda _: False
    lzma = None

# Each function accept a single location argument and return True if this is
# an archive
archive_handlers = [
    zipfile.is_zipfile,
    tarfile.is_tarfile,
    is_gzipfile,
    is_bz2file,
    is_lzmafile
]


def _can_extract(location):
    """
    Return True if this location is likely to be extractible as some archive
    or compressed file.
    """
    return any(handler(location) for handler in archive_handlers)


try:
    from extractcode.archive import can_extract  # NOQA
except ImportError:
    can_extract = _can_extract



# -*- coding: utf-8 -*-
"""
    pygments.lexers.other
    ~~~~~~~~~~~~~~~~~~~~~

    Just export lexer classes previously contained in this module.

    :copyright: Copyright 2006-2019 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

from typecode._vendor.pygments.lexers.sql import SqlLexer, MySqlLexer, SqliteConsoleLexer
from typecode._vendor.pygments.lexers.shell import BashLexer, BashSessionLexer, BatchLexer, \
    TcshLexer
from typecode._vendor.pygments.lexers.robotframework import RobotFrameworkLexer
from typecode._vendor.pygments.lexers.testing import GherkinLexer
from typecode._vendor.pygments.lexers.esoteric import BrainfuckLexer, BefungeLexer, RedcodeLexer
from typecode._vendor.pygments.lexers.prolog import LogtalkLexer
from typecode._vendor.pygments.lexers.snobol import SnobolLexer
from typecode._vendor.pygments.lexers.rebol import RebolLexer
from typecode._vendor.pygments.lexers.configs import KconfigLexer, Cfengine3Lexer
from typecode._vendor.pygments.lexers.modeling import ModelicaLexer
from typecode._vendor.pygments.lexers.scripting import AppleScriptLexer, MOOCodeLexer, \
    HybrisLexer
from typecode._vendor.pygments.lexers.graphics import PostScriptLexer, GnuplotLexer, \
    AsymptoteLexer, PovrayLexer
from typecode._vendor.pygments.lexers.business import ABAPLexer, OpenEdgeLexer, \
    GoodDataCLLexer, MaqlLexer
from typecode._vendor.pygments.lexers.automation import AutoItLexer, AutohotkeyLexer
from typecode._vendor.pygments.lexers.dsls import ProtoBufLexer, BroLexer, PuppetLexer, \
    MscgenLexer, VGLLexer
from typecode._vendor.pygments.lexers.basic import CbmBasicV2Lexer
from typecode._vendor.pygments.lexers.pawn import SourcePawnLexer, PawnLexer
from typecode._vendor.pygments.lexers.ecl import ECLLexer
from typecode._vendor.pygments.lexers.urbi import UrbiscriptLexer
from typecode._vendor.pygments.lexers.smalltalk import SmalltalkLexer, NewspeakLexer
from typecode._vendor.pygments.lexers.installers import NSISLexer, RPMSpecLexer
from typecode._vendor.pygments.lexers.textedit import AwkLexer
from typecode._vendor.pygments.lexers.smv import NuSMVLexer

__all__ = []



#
# Copyright (c) nexB Inc. and others.
# Visit https://github.com/nexB/typecode/
# Visit https://nexb.com https://aboutcode.org and
#  https://github.com/nexB/scancode-toolkit/ for support
# ScanCode is a trademark of nexB Inc.
#
# SPDX-License_identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from typecode.contenttype import get_type



#
# Copyright (c) nexB Inc. and others.
# Visit https://github.com/nexB/typecode/
# Visit https://nexb.com https://aboutcode.org and
#  https://github.com/nexB/scancode-toolkit/ for support
# ScanCode is a trademark of nexB Inc.
#
# SPDX-License_identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import os

from commoncode.testcase import FileBasedTesting

from typecode import extractible


class TestExtractible(FileBasedTesting):
    test_data_dir = os.path.join(os.path.dirname(__file__), 'data')

    def test__can_extract(self):
        tests = (
            ('extractible/a.tar.gz', True),
            ('extractible/crashing-squashfs', False),
            ('extractible/dbase.fdt', False),
            ('extractible/e.tar.bz2', True),
            ('extractible/e.tar.gz', True),
            ('extractible/e.tar', True),
            ('extractible/file_4.26-1.diff.gz', True),
            ('extractible/posixnotgnu.tar', True),
            ('extractible/sqfs-gz.sqs', False),
            ('extractible/sqfs-lzo.sqs', False),
            ('extractible/sqfs-xz.sqs', False),
            ('extractible/test.tar.lzma', True),
            ('extractible/test.tar.xz', True),
            ('extractible/test.zip', True),
            ('extractible/win-archive.lib', False),
        )
        for location, expected in tests:
            test_file = self.get_test_loc(location)
            result = extractible._can_extract(test_file)
            assert result == expected, '{} should extractible: {}'.format(location, expected)
